{% extends "admin/base.html" %}

{% block extrahead %}
    <script src="http://ajax.googleapis.com/ajax/libs/jquery/1.9.0/jquery.js" type="text/javascript"></script>
    {% include 'autocomplete_light/static.html' %}
    <style type="text/css">
    td.original { position:absolute; }
    </style>
{% endblock %}
